@@ -9,6 +9,7 @@
 from sqlalchemy import Column, Integer, VARCHAR, BigInteger, SmallInteger, DateTime
 from sqlalchemy import Text, ForeignKey, Float
 from sqlalchemy.orm import declarative_base, relationship
+from .registry import RekordboxAgentRegistry
 
 __all__ = [
     "AgentRegistry",
@@ -48,30 +49,25 @@ __all__ = [
     "ImageFile",
     "SettingFile",
     "UuidIDMap",
-    "get_update_count",
-    "reset_update_counts",
 ]
 
-__update_count__ = dict()
-
-
-def get_update_count(instance):
-    return __update_count__[instance]
-
-
-def reset_update_counts():
-    __update_count__.clear()
-
 
 # -- Base- and Mixin classes -----------------------------------------------------------
 
 
 class _Base(object):
 
-    __tablename__ = ""
+    __tablename__: str
 
-    def __init__(self):
-        super().__init__()
+    @classmethod
+    def create(cls, **kwargs):
+        RekordboxAgentRegistry.disable_tracking()
+        # noinspection PyArgumentList
+        self = cls(**kwargs)
+        RekordboxAgentRegistry.enable_tracking()
+
+        RekordboxAgentRegistry.on_create(self)
+        return self
 
     def __iter__(self):
         exc = "registry", "metadata"
@@ -89,13 +85,7 @@ class _Base(object):
     # noinspection PyUnresolvedReferences
     def __setattr__(self, key, value):
         if not key.startswith("_"):
-            # Increment entry in instance update count dictionary
-            k = self
-            try:
-                __update_count__[k] += 1
-            except KeyError:
-                __update_count__[k] = 1
-
+            RekordboxAgentRegistry.on_update(self, key, value)
         super().__setattr__(key, value)
 
     @classmethod
@@ -109,7 +99,7 @@ class _Base(object):
 
     def pformat(self, indent="   "):
         lines = [f"{self.__tablename__}"]
-        columns = self.__iter__()
+        columns = self.columns()
         w = max(len(col) for col in columns)
         for col in columns:
             lines.append(f"{indent}{col:<{w}} {self.__getitem__(col)}")
@@ -504,7 +494,7 @@ class DjmdHistory(Base, StatsFull):
 
     __tablename__ = "djmdHistory"
 
-    ID = Column(VARCHAR(255), primary_key=True)
+    ID = Column(VARCHAR(255), ForeignKey("djmdHistory.ParentID"), primary_key=True)
     Seq = Column(Integer, default=None)
     Name = Column(VARCHAR(255), default=None)
     Attribute = Column(Integer, default=None)
@@ -512,6 +502,8 @@ class DjmdHistory(Base, StatsFull):
     DateCreated = Column(VARCHAR(255), default=None)
 
     Songs = relationship("DjmdSongHistory", back_populates="History")
+    Children = relationship("DjmdHistory", foreign_keys=ParentID)
+    Parent = relationship("DjmdHistory", foreign_keys=ID)
 
     def __repr__(self):
         s = f"{self.ID: <2} Name={self.Name}"
@@ -535,13 +527,18 @@ class DjmdHotCueBanklist(Base, StatsFull):
 
     __tablename__ = "djmdHotCueBanklist"
 
-    ID = Column(VARCHAR(255), primary_key=True)
+    ID = Column(
+        VARCHAR(255), ForeignKey("djmdHotCueBanklist.ParentID"), primary_key=True
+    )
     Seq = Column(Integer, default=None)
     Name = Column(VARCHAR(255), default=None)
     ImagePath = Column(VARCHAR(255), default=None)
     Attribute = Column(Integer, default=None)
     ParentID = Column(VARCHAR(255), ForeignKey("djmdHotCueBanklist.ID"), default=None)
 
+    Children = relationship("DjmdHotCueBanklist", foreign_keys=ParentID)
+    Parent = relationship("DjmdHotCueBanklist", foreign_keys=ID)
+
     def __repr__(self):
         s = f"{self.ID: <2} Name={self.Name}"
         return f"<{self.__class__.__name__}({s})>"
@@ -637,13 +634,15 @@ class DjmdMyTag(Base, StatsFull):
 
     __tablename__ = "djmdMyTag"
 
-    ID = Column(VARCHAR(255), primary_key=True)
+    ID = Column(VARCHAR(255), ForeignKey("djmdMyTag.ParentID"), primary_key=True)
     Seq = Column(Integer, default=None)
     Name = Column(VARCHAR(255), default=None)
     Attribute = Column(Integer, default=None)
     ParentID = Column(VARCHAR(255), ForeignKey("djmdMyTag.ID"), default=None)
 
     MyTags = relationship("DjmdSongMyTag", back_populates="MyTag")
+    Children = relationship("DjmdMyTag", foreign_keys=ParentID)
+    Parent = relationship("DjmdMyTag", foreign_keys=ID)
 
     def __repr__(self):
         s = f"{self.ID: <2} Name={self.Name}"
@@ -667,15 +666,17 @@ class DjmdPlaylist(Base, StatsFull):
 
     __tablename__ = "djmdPlaylist"
 
-    ID = Column(VARCHAR(255), primary_key=True)
+    ID = Column(VARCHAR(255), ForeignKey("djmdPlaylist.ParentID"), primary_key=True)
     Seq = Column(Integer, default=None)
     Name = Column(VARCHAR(255), default=None)
     ImagePath = Column(VARCHAR(255), default=None)
     Attribute = Column(Integer, default=None)
-    ParentID = Column(VARCHAR(255), ForeignKey("djmdMyTag.ID"), default=None)
+    ParentID = Column(VARCHAR(255), ForeignKey("djmdPlaylist.ID"), default=None)
     SmartList = Column(Text, default=None)
 
     Songs = relationship("DjmdSongPlaylist", back_populates="Playlist")
+    Children = relationship("DjmdPlaylist", foreign_keys=ParentID)
+    Parent = relationship("DjmdPlaylist", foreign_keys=ID)
 
     def __repr__(self):
         s = f"{self.ID: <2} Name={self.Name}"
@@ -699,7 +700,9 @@ class DjmdRelatedTracks(Base, StatsFull):
 
     __tablename__ = "djmdRelatedTracks"
 
-    ID = Column(VARCHAR(255), primary_key=True)
+    ID = Column(
+        VARCHAR(255), ForeignKey("djmdRelatedTracks.ParentID"), primary_key=True
+    )
     Seq = Column(Integer, default=None)
     Name = Column(VARCHAR(255), default=None)
     Attribute = Column(Integer, default=None)
@@ -707,6 +710,8 @@ class DjmdRelatedTracks(Base, StatsFull):
     Criteria = Column(Text, default=None)
 
     Songs = relationship("DjmdSongRelatedTracks", back_populates="RelatedTracks")
+    Children = relationship("DjmdRelatedTracks", foreign_keys=ParentID)
+    Parent = relationship("DjmdRelatedTracks", foreign_keys=ID)
 
     def __repr__(self):
         s = f"{self.ID: <2} Name={self.Name}"
@@ -732,13 +737,15 @@ class DjmdSampler(Base, StatsFull):
 
     __tablename__ = "djmdSampler"
 
-    ID = Column(VARCHAR(255), primary_key=True)
+    ID = Column(VARCHAR(255), ForeignKey("djmdSampler.ID"), primary_key=True)
     Seq = Column(Integer, default=None)
     Name = Column(VARCHAR(255), default=None)
     Attribute = Column(Integer, default=None)
     ParentID = Column(VARCHAR(255), ForeignKey("djmdSampler.ID"), default=None)
 
     Songs = relationship("DjmdSongSampler", back_populates="Sampler")
+    Children = relationship("DjmdSampler", foreign_keys=ParentID)
+    Parent = relationship("DjmdSampler", foreign_keys=ID)
 
     def __repr__(self):
         s = f"{self.ID: <2} Name={self.Name}"
